#!/usr/bin/env python3

"""Show test failure for high Unicode characters with encoding overridden.
"""

import os
import platform
import re
import socket
import sys
import time
import pyodbc

VALUE = "x \U0001F31C z"
PORT = 5432
CONNECTION_STRING = (
    "Driver={PostgreSQL Unicode};"
    "Server=db;"
    "Uid=docker;"
    "Database=docker;"
    "Pwd=docker;"
)


def wait_for_database():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    while True:
        try:
            sock.connect(("db", PORT))
            sock.close()
            return
        except socket.error:
            time.sleep(0.1)


def test(label, conn):
    sys_version = re.sub(r"\s+", " ", sys.version)
    driver_name = conn.getinfo(pyodbc.SQL_DRIVER_NAME)
    driver_version = conn.getinfo(pyodbc.SQL_DRIVER_VER)
    supported_odbc_version = conn.getinfo(pyodbc.SQL_DRIVER_ODBC_VER)
    unicode_size = pyodbc.UNICODE_SIZE
    sqlwchar_size = pyodbc.SQLWCHAR_SIZE
    print("=" * 60)
    print(f"with {label}")
    print("=" * 60)
    print(f"python:  {sys_version}")
    print(f"pyodbc:  {pyodbc.version}")
    print(f"odbc:    {conn.getinfo(pyodbc.SQL_ODBC_VER)}")
    print(f"driver:  {driver_name} {driver_version}")
    print(f"         supports ODBC version {supported_odbc_version}")
    print(f"os:      {platform.system()}")
    print(f"unicode: Py_Unicode={unicode_size} SQLWCHAR={sqlwchar_size}")
    cursor = conn.cursor()
    cursor.execute("CREATE TABLE t1(s VARCHAR(100))")
    cursor.execute(f"INSERT INTO t1 VALUES ('{VALUE}')")
    result = cursor.execute("SELECT s FROM t1").fetchone()[0]
    print("-" * 60)
    if result == VALUE:
        print(f"{result!r} matches the expected value {VALUE!r}")
    else:
        print(f"{result!r} does not match the expected value {VALUE!r}")
    print()


wait_for_database()
conn = pyodbc.connect(CONNECTION_STRING)
test("default encoding", conn)
conn = pyodbc.connect(CONNECTION_STRING)
conn.setencoding(encoding="utf-8")
test("encoding overridden", conn)
